<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:variable name="smallcase"
                select="'abcdefghijklmnopqrstuvwxyz'"/>
  <xsl:variable name="uppercase"
                select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>
  <xsl:strip-space elements="*"/>
  <xsl:output method="xml"
              indent="yes"
              encoding="utf-8" />
  <xsl:template match="Config">
    <monitorConfiguration xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                          xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                          xmlns="http://www.bidib.org/schema/monitor/configuration/1.1">
      <globalConfiguration>
        <xsl:attribute name="serialPort">
          <xsl:value-of select="translate(Parameter[@Name='Comport']/@Value,$smallcase,$uppercase)"/>
        </xsl:attribute>
        <xsl:attribute name="baudrate">
          <xsl:value-of select="Parameter[@Name='Baudrate']/@Value"/>
        </xsl:attribute>
        <xsl:attribute name="bidibDataDirectory">
          <xsl:value-of select="Parameter[@Name='BiDiBDataDir']/@Value"/>
        </xsl:attribute>
        <xsl:attribute name="userDataDirectory">
          <xsl:value-of select="Parameter[@Name='UserDataDir']/@Value"/>
        </xsl:attribute>
        <xsl:attribute name="devMode">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='DevMode']/@Value='1'">true</xsl:when>
            <xsl:when test="Parameter[@Name='DevMode']/@Value='0'">false</xsl:when>
            <xsl:otherwise><xsl:value-of select="translate(Parameter[@Name='DevMode']/@Value,$uppercase,$smallcase)"/></xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="showSystemLog">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='LogWindow']/@Value='1'">true</xsl:when>
            <xsl:when test="Parameter[@Name='LogWindow']/@Value='0'">false</xsl:when>
            <xsl:otherwise><xsl:value-of select="translate(Parameter[@Name='ShowSystemLog']/@Value,$uppercase,$smallcase)"/></xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="technicalView">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='TechnicalView']/@Value='1'">true</xsl:when>
            <xsl:when test="Parameter[@Name='TechnicalView']/@Value='0'">false</xsl:when>
            <xsl:otherwise><xsl:value-of select="translate(Parameter[@Name='TechnicalView']/@Value,$uppercase,$smallcase)"/></xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="rewriteUsername">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='ReWriteUsername']/@Value='1'">true</xsl:when>
            <xsl:when test="Parameter[@Name='RewriteUsername']/@Value='1'">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="watchdogActive">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='WatchdogActive']/@Value='1'">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="initSpeedToZero">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='InitSpeedToZero']/@Value='1'">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="boosterAndDccOff">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='BoosterAndDccOFF']/@Value='1'">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="resetNodesTreeOnDisconnect">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='NodeTreeDelete']/@Value='1'">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="sendDccToAll">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='SendDCC']/@Value='1'">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="clockRunEndless">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='ClockEndless']/@Value='1'">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
         <xsl:attribute name="clockAccelerationFactor">
          <xsl:value-of select="Parameter[@Name='ClockFaktor']/@Value"/>
        </xsl:attribute>
         <xsl:attribute name="clockWeekday">
          <xsl:value-of select="Parameter[@Name='ClockWeekday']/@Value"/>
        </xsl:attribute>
        <xsl:attribute name="connectionType">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='InterfaceTechnology']/@Value='2'">NetworkTcp</xsl:when>
            <xsl:otherwise>SerialPort</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="language">
          <xsl:choose>
            <xsl:when test="Parameter[@Name='Language']">
              <xsl:value-of select="Parameter[@Name='Language']/@Value"/>
            </xsl:when>
            <xsl:otherwise>de</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="nodeTabWaitTime">
          <xsl:value-of select="Parameter[@Name='NodeTabWait']/@Value"/>
        </xsl:attribute>
        <xsl:attribute name="networkHostAddress">
          <xsl:value-of select="Parameter[@Name='Interface_IPAdress']/@Value"/>
        </xsl:attribute>
        <xsl:attribute name="numberOfDriveControls">
          <xsl:value-of select="Parameter[@Name='NumberOfLokControls']/@Value"/>
        </xsl:attribute>
        <xsl:attribute name="clockStartTime">
          <xsl:value-of select="concat(Parameter[@Name='ClockStartHour']/@Value,':' ,Parameter[@Name='ClockStartMin']/@Value)"/>
        </xsl:attribute>
        <xsl:attribute name="clockEndTime">
          <xsl:value-of select="concat(Parameter[@Name='ClockEndeHour']/@Value,':' ,Parameter[@Name='ClockEndeMin']/@Value)"/>
        </xsl:attribute>
        <xsl:attribute name="dccGeneratorId">
          <xsl:if test="not(Parameter[@Name='DCCGeneratorHexUID']/@Value='-')">
            <xsl:value-of select="Parameter[@Name='DCCGeneratorHexUID']/@Value"/>
          </xsl:if>    
        </xsl:attribute>
        <!-- add new defaults -->
        <xsl:attribute name="activeLogLevels">
        <xsl:choose>
            <xsl:when test="Parameter[@Name='ActiveLogLevels']">
              <xsl:value-of select="Parameter[@Name='ActiveLogLevels']/@Value"/>
            </xsl:when>
            <xsl:otherwise>Info,Warn,Error,Fatal</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
      </globalConfiguration>
    </monitorConfiguration>
  </xsl:template>
</xsl:stylesheet>
