<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.bidib.org/jbidibc/simulation/nodes" xmlns:bd="http://www.bidib.org/schema/bidib/2.0">
	<xsl:variable name="smallcase" select="'abcdefghijklmnopqrstuvwxyz'"/>
	<xsl:variable name="uppercase" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>
	<xsl:strip-space elements="*"/>
	<xsl:output method="xml" indent="yes" encoding="utf-8"/>
	<xsl:template name="decimalToHex">
		<xsl:param name="dec"/>
		<xsl:if test="$dec > 0">
			<xsl:call-template name="decimalToHex">
				<xsl:with-param name="dec" select="floor($dec div 16)"/>
			</xsl:call-template>
			<xsl:value-of select="substring('0123456789ABCDEF', (($dec mod 16) + 1), 1)"/>
		</xsl:if>
	</xsl:template>
	<xsl:template match="/">
		<simulation xmlns="http://www.bidib.org/jbidibc/simulation/nodes" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.bidib.org/jbidibc/simulation/nodes /xsd/simulation.xsd">
			<xsl:apply-templates select="ArrayOfBiDiBNode/BiDiBNode[1]"/>
		</simulation>
	</xsl:template>
	<xsl:template match="BiDiBNode[1]">
		<master>
			<xsl:attribute name="uniqueId">
				<xsl:call-template name="decimalToHex">
					<xsl:with-param name="dec">
						<xsl:value-of select="@uniqueId"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:attribute name="address">0</xsl:attribute>
			<xsl:attribute name="className">org.bidib.jbidibc.simulation.nodes.DefaultNodeSimulator</xsl:attribute>
			<xsl:attribute name="userName">
				<xsl:value-of select="FullUserName/@Value"/>
			</xsl:attribute>
			<xsl:apply-templates select="bd:features"/>
			<subNodes>
				<xsl:apply-templates select="parent::node()/BiDiBNode[position() > 1]"/>
			</subNodes>
		</master>
	</xsl:template>
	<xsl:template match="BiDiBNode[position()&gt;1]">
		<node>
			<xsl:attribute name="uniqueId">
				<xsl:call-template name="decimalToHex">
					<xsl:with-param name="dec">
						<xsl:value-of select="@uniqueId"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:attribute name="address">
				<xsl:value-of select="count(preceding-sibling::node())"/>
			</xsl:attribute>
			<xsl:attribute name="className">org.bidib.jbidibc.simulation.nodes.DefaultNodeSimulator</xsl:attribute>
			<xsl:attribute name="userName">
				<xsl:value-of select="FullUserName"/>
			</xsl:attribute>
			<xsl:apply-templates select="bd:features"/>
		</node>
	</xsl:template>
	<xsl:template match="bd:features">
		<Features>
			<xsl:apply-templates select="bd:feature"/>
		</Features>
	</xsl:template>
	<xsl:template match="bd:feature">
		<feature>
			<xsl:attribute name="type">
				<xsl:value-of select="@featureCodeId"/>
			</xsl:attribute>
			<xsl:attribute name="value">
				<xsl:value-of select="@value"/>
			</xsl:attribute>
		</feature>
	</xsl:template>
</xsl:stylesheet>