<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
xmlns="http://www.bidib.org/schema/userDevices/1.2" xmlns:ct="http://www.bidib.org/schema/commonTypes/1.0" 
xmlns:MoExt="urn:MoExt">
	<xsl:variable name="smallcase" select="'abcdefghijklmnopqrstuvwxyz'"/>
	<xsl:variable name="uppercase" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>
	<xsl:output method="xml" indent="yes" encoding="utf-8"/>
	<xsl:template match="UserDevice">
		<userDevicesList>
			<version>
				<xsl:attribute name="lastUpdate">
					<xsl:value-of select="MoExt:NewDateTime()"/>
				</xsl:attribute>
				<xsl:attribute name="created">
					<xsl:value-of select="MoExt:NewDate()"/>
				</xsl:attribute>
			</version>
			<userDevices>
				<xsl:apply-templates/>
			</userDevices>
		</userDevicesList>
	</xsl:template>
	<xsl:template match="Decoder">
		<userDevice>
			<xsl:attribute name="id">
				<xsl:value-of select="MoExt:NewGuid()"/>
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="text()"/>
			</xsl:attribute>
			<xsl:attribute name="address">
				<xsl:value-of select="@PomAddress"/>
			</xsl:attribute>
			<xsl:attribute name="image">
				<xsl:value-of select="Data/@Image"/>
			</xsl:attribute>
			<xsl:attribute name="deviceType">
				<xsl:value-of select="translate(@DecoderType,$uppercase,$smallcase)"/>
			</xsl:attribute>
			<xsl:attribute name="articleNumber">
				<xsl:value-of select="Data/@ArtNo"/>
			</xsl:attribute>
			<xsl:attribute name="manufacturer">
				<xsl:value-of select="Data/@Manufacturer"/>
			</xsl:attribute>
			<xsl:attribute name="comment">
				<xsl:value-of select="Data/@Comment"/>
			</xsl:attribute>
			<xsl:attribute name="price">
				<xsl:value-of select="Data/@Price"/>
			</xsl:attribute>
			<xsl:attribute name="purchaseDate">
				<xsl:value-of select="Data/@BoughtDate"/>
			</xsl:attribute>
			<xsl:attribute name="owner">
				<xsl:value-of select="Data/@Owner"/>
			</xsl:attribute>
			<xsl:attribute name="epoch">
				<xsl:choose>
					<xsl:when test="Data/@Epoch='1'">1</xsl:when>
					<xsl:when test="Data/@Epoch='I'">1</xsl:when>
					<xsl:when test="Data/@Epoch='2'">2</xsl:when>
					<xsl:when test="Data/@Epoch='II'">2</xsl:when>
					<xsl:when test="Data/@Epoch='3'">3</xsl:when>
					<xsl:when test="Data/@Epoch='III'">3</xsl:when>
					<xsl:when test="Data/@Epoch='4'">4</xsl:when>
					<xsl:when test="Data/@Epoch='IV'">4</xsl:when>
					<xsl:when test="Data/@Epoch='5'">5</xsl:when>
					<xsl:when test="Data/@Epoch='V'">5</xsl:when>
					<xsl:when test="Data/@Epoch='6'">6</xsl:when>
					<xsl:when test="Data/@Epoch='VI'">6</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<decoder>
				<xsl:attribute name="name">
					<xsl:value-of select="@DecoderName"/>
				</xsl:attribute>
				<xsl:attribute name="decoderId">
					<xsl:value-of select="@DID"/>
				</xsl:attribute>
				<xsl:attribute name="speedsteps">
					<xsl:value-of select="@Speedsteps"/>
				</xsl:attribute>
				<xsl:attribute name="manufacturerId">
					<xsl:value-of select="@VendorID"/>
				</xsl:attribute>
				<xsl:attribute name="manufacturerExtendedId">0</xsl:attribute>
			</decoder>
		</userDevice>
	</xsl:template>
</xsl:stylesheet>
